<?php

/**
 * The awesome Blog-Libre.org Google Fonts Plugin
 *
 * Download and store locally fonts used by WordPress from Google.
 *
 * @package BL_Google_Fonts
 * @subpackage Main
 */

/**
 * Plugin Name: BL Google Fonts
 * Plugin URI:  http://www.blog-libre.org
 * Description: Download and store locally fonts used by WordPress from Google. An awesome Blog-Libre.org worldwide exclusivity ! :-)
 * Author:      Laurent NAPIAS
 * Author URI:  http://www.blog-libre.org
 * Version:     1.0
 * License:     GPL
 */

/* Exit if accessed directly */
if ( ! defined( 'ABSPATH' ) ) exit;

class blGoogleFonts {
	/**
	 * Hook filters.
	 * 
	 * @since 1.0
	 * @access public
	 */
	public function __construct() {
		add_filter( 'style_loader_src', array( $this, 'changeGoogleFonts'), 888, 4 );
	}

	/**
	 * Change url and download fonts
	 *
	 * @since 1.0
	 * @access public
	 *
	 * @param  string $src original url of the loaded css file
	 * @return string $src new url for the modified css file
	 */
	public function changeGoogleFonts( $src ) {

		// Si la feuille de style liée provient de googleapis
		if (preg_match("(fonts\.googleapis\.com([^<].+))", $src, $match)) {

				// $match[0] contient l'url complète et on traîte ces feuille de style
				$newCssUrl = $this->handdleFromGoogleApis($match[0]);
				
				// Si il n'y a pas d'erreur, les fichiers de polices ont été téléchargés la feuille de style modifiée
				// On donne donc l'url de la nouvelle feuille de styles
				if ($newCssUrl != false) {
						$src = $newCssUrl;
				}
		}

		// On retourne l'url d'origine de la feuille de styles en cas d'erreur de traitement
		return $src;
	}

	/**
	 * Process url and download fonts from Google
	 *
	 * @since 1.0
	 * @access public
	 *
	 * @param  string $url original url to Google API servers
	 * @return boolean true if no error during process
	 */
	public function handdleFromGoogleApis($url) {

		// Il manque 'http://' dans l'url
		$url = 'http://'.$url;

		// On va chercher le nom de la police
		$patterns = array(
					// On remplace le début de l'url
		'!^http://fonts.googleapis.com/css\?!',
					// On capture la famille en ignorant d'éventuels attributs
		'!(family=[^&:]+).*$!',
					// On supprime le nom de la variable dans l'url
		'!family=!',
					// On remplace un éventuel signe '+'
		'!\+!');
		$replacements = array(
		"",
		'$1',
		'',
		' ');
		
		// On veut le nom de la police est en miniatures avec '-' à la place de ' '
		// L'url doît être décodée car des variables sont passées dans l'url
		$localCssName = strtolower(str_replace(' ', '-', preg_replace($patterns, $replacements, urldecode($url))));
		
		// On rajoute l'extension à notre fichier local et détermine l'emplacement où il doit être
		$localCssFilename = plugin_dir_path( __FILE__ ).'css/'. $localCssName.'.css';
		
		// Le fichier local n'existe pas alors on va télécharger la feuille de style depuis Google,
		// télécharger les polices, modifier la feuille de style et l'enregistrer
		if (!file_exists($localCssFilename)) {
				if ($this->updateCssFile($url, $localCssName)) {
						// On rajoute l'extension à notre fichier local et détermine l'url où il doit être
						return plugin_dir_url( __FILE__ ).'css/'. $localCssName.'.css';
				}
		} else {
			// On rajoute l'extension à notre fichier local déjà existant et détermine l'url où il doit être
			return plugin_dir_url( __FILE__ ).'css/'. $localCssName.'.css';
		}

		// Retour avec erreur
		return false;
	}

	/**
	 * Change original css file content and url, download fonts from Google
	 *
	 * @since 1.0
	 * @access public
	 *
	 * @param  string $url original url to Google API servers
	 * @param  string $localCssName the local css name of this font
	 * @return boolean true if no error during process
	 */
	public function updateCssFile($url, $localCssName) {
		// On récupère la feuille de styles depuis Google
		$response = wp_remote_get($url);
		
		/* On obtient par exemple
		@font-face {
			font-family: 'Open Sans';
			font-style: normal;
			font-weight: 300;
			src: local('Open Sans Light'), local('OpenSans-Light'), url(http://fonts.gstatic.com/s/opensans/v11/DXI1ORHCpsQm3Vp6mXoaTdqQynqKV_9Plp7mupa0S4g.ttf) format('truetype');
		}
		...
		
		On va changer les urls par des urls venant de notre serveur et enregistrant les fichiers de police
		sous le nom donné par le second local('...') qui n'est pas toujours présent
		On utilisera le premier dans ce cas.
		*/

		// On vérifie que la réponse a bien été traitée sans erreur
		if ( ! is_wp_error( $response ) ) {
			
				// La feuille de styles depuis Google est dans $response['body']
				$cssFileContent = $response['body'];
				
				// On récupère La liste de tous les fichiers de polices à télécharger avec leur propre nom
				$filesToDownload = $this->getFilesToDownload($response);
		
				// L'emplacement local pour stocker les fichiers de police
				$localFontPath = plugin_dir_path( __FILE__ ).'fonts/'.$localCssName;
				
				// On crée ce dossier
				if (wp_mkdir_p( $localFontPath )) {
						
						foreach($filesToDownload as $fileToDownload) {
							// On télécharge et stocke chaque fichier de police
							$response = wp_remote_get($fileToDownload->url);

							// On vérifie que la réponse a bien été traitée sans erreur
							if ( ! is_wp_error( $response ) ) {
									// Le fichier de police téléchargé est dans $response['body']
									$fontFile = $response['body'];

									$localFontFilename = $localFontPath.'/'.$fileToDownload->filename;

									// On enregsitre le fichier
									$f = fopen($localFontFilename,'wb');
									fwrite($f, $fontFile, strlen($fontFile));
									fclose($f);
				
									// On change la feuille de style pour faire référence au fichier local
									$cssFileContent = str_replace($fileToDownload->url, 
																								'../fonts/'.$localCssName.'/'.$fileToDownload->filename,
																								$cssFileContent);
							}
						}

						// On rajoute l'extension à notre fichier local et détermine l'emplacement où il doit être
						$localCssFilename = plugin_dir_path( __FILE__ ).'/css/'. $localCssName.'.css';
								
						// On enregsitre la feuille de style modifiée
						$f = fopen($localCssFilename,'wb');
						fwrite($f, $cssFileContent, strlen($cssFileContent));
						fclose($f);
						
						// Retour sans erreur
						return true;
				}
		}

		// Retour avec erreur
		return false;
	}

	/**
	 * Handdle original css file content to know filenames for the fonts to download from Google
	 *
	 * @since 1.0
	 * @access public
	 *
	 * @param  string $response an wp_remote_get object
	 * @return boolean true if no error during process
	 */
	public function getFilesToDownload($response) {

		// La feuille de styles depuis Google est dans $response['body']
		$cssFileContent = $response['body'];

		// On récupère les différents informations que l'on souhaite dans des tableaux
		preg_match_all("/src\:[\s]?(local\('[a-zA-Z0-9\-\s]+'\)),([\s\n\t\r]?local\('[a-zA-Z0-9\-\s]+'\),)?[\s\n\t\r]?url\([\s]?[\"|\']?(.*?)([\"|\'])?\)/i", $cssFileContent, $matches, PREG_PATTERN_ORDER);
		
		// On renomme les fichies pour ne pas se perdre ;-)
		// $matches[0] contient un tableau de toutes les lignes 'src: local(...' inutile ici
		$fisrstLocalNames = $matches[1];
		$secondLocalNames = $matches[2];
		$fontUrls = $matches[3];

		// La liste de tous les fichiers de polices à télécharger avec leur propre nom
		$filesToDownload = array();
		
		// On absolument besoin de la clé pour se simplifier la vie
		foreach($secondLocalNames as $key => $values)
		{
			// On crée une classe
			$fileToDownload = new StdClass();
			
			// Le second local('...') est manquant, alors on prend le premier via la clé
			if ($value == '')
			{
				// Par exemple : local('Open Sans Light'), on sépare via ' dans un tableau 
				list($start, $filename, $end) = explode("'", $fisrstLocalNames[$key]);
			}
			else
			{
				// Sinon, on prend le second
				list($start, $filename, $end) = explode("'", $secondLocalNames[$key]);
			}

			// On change les ' ' par des '-' et on ajoute un point
			$filename = str_replace(' ', '-', $filename).'.';

			// L'extension du fichier de la police est donnée par son url
			$fileToDownload->filename = $filename.pathinfo(parse_url($fontUrls[$key],PHP_URL_PATH),PATHINFO_EXTENSION);
			$fileToDownload->url = $fontUrls[$key];
			
			// On ajoute ce fichier à télécharger à notre tableau
			$filesToDownload[$key] = $fileToDownload;
		}
	
		return $filesToDownload;
	}
}

/* Although it would be preferred to do this on hook,
 * load early to make sure 'Open Sans' is downloaded
 */
$blGoogleFonts = new blGoogleFonts();
